// async_reader - A simple buffered input source.

#pragma once

#include <queue>
#include <functional>

#include <boost/asio.hpp>

/* API -- this is what we're working with */

/// readSomeInts
/// @brief Get some integers from input
/// @param buf - Buffer to put the data into
/// @param max - Won't put more than this many into the buffer
/// @returns the number of ints that were read in
size_t readSomeInts(int buf[], size_t max);

/******************************************/

/// @class AsyncReader
/// @brief a buffered input source with no concurrency
///
/// Enables us to get ints from our input one-at-a-time. Uses readSomeInts
class AsyncReader
{
public:
    /// @param cb: A callback for what to do with the int
    void getInt(std::function<void(int)> cb);

	// go ahead and start filling up the buffer immediately 
    AsyncReader(boost::asio::io_context& ctx) : m_strand(boost::asio::make_strand(ctx)) { doReadSome([](){}); }

private:
    static constexpr size_t c_max_buffer_size = 10;
    std::queue<int> m_buffer;

    boost::asio::strand<boost::asio::any_io_executor> m_strand;

    void doReadSome(std::function<void()> cb);

};
